@@ -706,10 +706,15 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		bool foundp;
 		POOL_STATUS status;
 		char *search_query = NULL;
+		int len;
+		char *tmp;
+#define STR_ALLOC_SIZE 1024
 
-		search_query = (char *)malloc(sizeof(char) * strlen(query) + 1);
+		len = strlen(query)+1;
+		search_query = (char *)malloc(len);
 		if (search_query == NULL)
 		{
+			pool_error("Execute: malloc failed");
 			return POOL_END;
 		}
 		strcpy(search_query, query);
@@ -722,26 +727,36 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			/* Extract binary contents from bind message */
 			char *query_in_bind_msg = bind_msg->contents + bind_msg->param_offset;
 			char hex_str[4];  /* 02X chars + white space + null end */
-			int max_search_query_size = 0;
-			int i = 0;
-			char *tmp;
+			int i;
+			int alloc_len;
+
+			alloc_len = (len/STR_ALLOC_SIZE+1)*STR_ALLOC_SIZE;
+			search_query = realloc(search_query, alloc_len);
+			if (search_query == NULL)
+			{
+				pool_error("Execute: realloc failed");
+				return POOL_END;
+			}
 
 			for (i = 0; i < bind_msg->len - bind_msg->param_offset; i++)
 			{
-				while (max_search_query_size <= sizeof(char) * strlen(search_query))
-				{
-					max_search_query_size += 1024;
-					tmp = (char *)realloc(search_query, sizeof(char) * max_search_query_size);
-					if (tmp == NULL)
-					{
-						return POOL_END;
-					}
-					search_query = tmp;
-					tmp = NULL;
-				}
+				int hexlen;
 
 				snprintf(hex_str, sizeof(hex_str), (i == 0) ? " %02X" : "%02X", 0xff & query_in_bind_msg[i]);
+				hexlen = strlen(hex_str);
+
+				if ((len+hexlen) > alloc_len)
+				{
+					alloc_len += STR_ALLOC_SIZE;
+					search_query = realloc(search_query, alloc_len);
+					if (search_query == NULL)
+					{
+						pool_error("Execute: realloc failed");
+						return POOL_END;
+					}
+				}
 				strcat(search_query, hex_str);
+				len += hexlen;
 			}
 
 			query_context->query_w_hex = search_query;
@@ -752,15 +767,25 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			 * So overwrite the query text in temp cache to the one with the hex of bind message.
 			 * If not, md5 hash will be created by the query text without bind message, and
 			 * it will happen to find cache never or to get a wrong result.
+			 * 
+			 * However, It is possible that temp_cache does not exist.
+			 * Consider following scenario:
+			 * - In the previous execute cache is overflowed, and
+			 *   temp_cache discarded.
+			 * - In the subsequent bind/execute uses the same portal
 			 */
-			tmp = (char *)malloc(sizeof(char) * strlen(search_query) + 1);
-			if (tmp == NULL)
+			if (query_context->temp_cache)
 			{
-				return POOL_END;
+				tmp = (char *)malloc(sizeof(char) * strlen(search_query) + 1);
+				if (tmp == NULL)
+				{
+					pool_error("Execute: malloc failed");
+					return POOL_END;
+				}
+				free(query_context->temp_cache->query);
+				query_context->temp_cache->query = tmp;
+				strcpy(query_context->temp_cache->query, search_query);
 			}
-			free(query_context->temp_cache->query);
-			query_context->temp_cache->query = tmp;
-			strcpy(query_context->temp_cache->query, search_query);
 		}
 
 		/* If the query is SELECT from table to cache, try to fetch cached result. */
